@@ -29,11 +29,13 @@
 
 #include "camel-exchange-store.h"
 #include "camel-exchange-folder.h"
+#include "camel-exchange-summary.h"
 #include <camel/camel-session.h>
+#include <camel/camel-url.h>
 
 #define SUBFOLDER_DIR_NAME     "subfolders"
 #define SUBFOLDER_DIR_NAME_LEN 10
-#define d(x)
+#define d(x) 
 
 
 //static CamelStoreClass *parent_class = NULL;
@@ -499,23 +501,28 @@ exchange_get_folder (CamelStore *store, const char *folder_name,
 		g_free (folder_dir);
 		return folder;
 	}
-	g_mutex_unlock (exch->folders_lock);
 
 	folder = (CamelFolder *)camel_object_new (CAMEL_EXCHANGE_FOLDER_TYPE);
+	g_hash_table_insert (exch->folders, g_strdup (folder_name), folder);
+	g_mutex_unlock (exch->folders_lock);
 	
 	if (!camel_exchange_folder_construct (folder, store, folder_name,
 					      flags, folder_dir, ((CamelOfflineStore *) store)->state,
 					      exch->stub, ex)) {
+		gchar *key;
+		g_mutex_lock (exch->folders_lock);
+		if (g_hash_table_lookup_extended (exch->folders, folder_name,
+						  (gpointer *) &key, NULL)) {
+			g_hash_table_remove (exch->folders, key);
+			g_free (key);
+		}
+		g_mutex_unlock (exch->folders_lock);
 		g_free (folder_dir);
 		camel_object_unref (CAMEL_OBJECT (folder));
 		return NULL;
 	}
 	g_free (folder_dir);
 
-	g_mutex_lock (exch->folders_lock);
-	g_hash_table_insert (exch->folders, g_strdup (folder_name), folder);
-	g_mutex_unlock (exch->folders_lock);
-
 	/* If you move messages into a folder you haven't visited yet, it
 	 * may create and then unref the folder. That's a waste. So don't
 	 * let that happen. Probably not the best fix... 
@@ -608,18 +615,44 @@ make_folder_info (CamelExchangeStore *exch, char *name, char *uri,
 	d(printf ("make folder info : %s flags : %d\n", name, flags));
 	CamelFolderInfo *info;
 	const char *path;
+	gchar **components;
+	char *new_uri;
+	char *temp;
 
 	path = strstr (uri, "://");
 	if (!path)
 		return NULL;
-	path = strchr (path + 3, '/');
+	path = strstr (path + 3, "/;");
 	if (!path)
 		return NULL;
 
+	components = g_strsplit (uri, "/;", 2);
+	if (components[0] && components[1])
+		new_uri = g_strdup_printf ("%s/%s", components[0], components[1]);
+	else
+		new_uri = g_strdup (uri);
+	g_strfreev (components);
+
+	d(printf ("new_uri is : %s\n", new_uri));
 	info = g_new0 (CamelFolderInfo, 1);
 	info->name = name;
-	info->uri = uri;
-	info->full_name = g_strdup (path + 1);
+	info->uri = new_uri;
+
+#if 0	
+	/* Process the full-path and decode if required */
+	temp = strrchr (path+2, '/');
+	if (temp) {
+		/* info->full_name should not have encoded path */
+		/*info->full_name = camel_url_decode_path (path+2);*/
+	} else {
+		/* If there are no sub-directories, decoded(name) will be 
+		   equal to that of path+2.
+		   Ex: personal
+		*/
+		info->full_name = g_strdup (path+2);
+	}
+#endif
+	info->full_name = g_strdup (path+2);
 	info->unread = unread_count;
 
 	if (flags & CAMEL_STUB_FOLDER_NOSELECT)
@@ -1056,6 +1089,27 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		g_free (reninfo.old_base);
 		break;
 	}
+	
+	case CAMEL_STUB_RETVAL_FOLDER_SET_READONLY:
+	{
+		CamelFolder *folder;
+		gchar *folder_name;
+		guint32 readonly;
+
+		if (camel_stub_marshal_decode_folder (stub->status, &folder_name) == -1 ||
+		    camel_stub_marshal_decode_uint32 (stub->status, &readonly) == -1)
+			break;
+
+		g_mutex_lock (exch->folders_lock);
+		folder = g_hash_table_lookup (exch->folders, folder_name);
+		g_mutex_unlock (exch->folders_lock);
+		if (folder) {
+			camel_exchange_summary_set_readonly (folder->summary, readonly ? TRUE : FALSE);
+		}
+
+		g_free (folder_name);
+		break;
+	}
 
 	default:
 		g_assert_not_reached ();
