@@ -65,7 +65,8 @@ typedef enum {
 	CAMEL_STUB_RETVAL_THAW_FOLDER,
 	CAMEL_STUB_RETVAL_FOLDER_CREATED,
 	CAMEL_STUB_RETVAL_FOLDER_DELETED,
-	CAMEL_STUB_RETVAL_FOLDER_RENAMED
+	CAMEL_STUB_RETVAL_FOLDER_RENAMED,
+	CAMEL_STUB_RETVAL_FOLDER_SET_READONLY
 } CamelStubRetval;
 
 typedef enum {
